# 微信公众号推送模板消息
## 微信公众号基本配置
如图所示，点击微信公众号里的基本配置，拿到AppID和AppSecret

![微信公众号的AppID和AppSecret](https://upload-images.jianshu.io/upload_images/10171495-a5f2926d3852a783.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/800)

填写模板消息：

![微信截图_20180727165945.png](https://upload-images.jianshu.io/upload_images/10171495-4cb853a981f1367b.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/800)

将以上信息填入Config.py
```
# wechat
self.appid = ''       # AppID
self.secrect = ''      # Secret

# template_id
self.template_id = ""  # 填上自己的模板ID
```

##  获取access_token
通过appid和secret从微信公众号中获取access_token，运行以下代码即可输出access_token.
```
import requests
import json
import time
from Config import Config
from check_config import CheckConfig

cc = CheckConfig()
ret, msg = cc.check_all()
if ret != RET_OK:
    print(ret, msg)
    exit()
config = Config()

# 获取accessToken
class WechatPush(object):
    
    def __init__(self):
        self.app_id = config.appid
        self.secret = config.secrect
        self.access_generate_time = 0
        self.access_token = None
    def get_access_token_from_wechat(self):
        access_token_json = requests.get(
            url="https://api.weixin.qq.com/cgi-bin/token",
            params={
                "grant_type": "client_credential",
                "appid": self.app_id,
                "secret": self.secret
            }
        ).json()

        if 'errmsg' in access_token_json:
            print(access_token_json)
            return RET_ERROR, "Incorrect config, please check appid and secret. Or your ip did not add to whitelist."

        return RET_OK, access_token_json['access_token']

if __name__ == '__main__':
    print(wp.get_access_token())

```
## 推送模板消息（主动）
**接口调用请求**
> http请求方式: POST
>
> https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=ACCESS_TOKEN
>
> 这里需要填上自己的ACCESS_TOKEN

**POST数据格式**
```
     {
        "first": {
           "value":(msg['echo_type']),
           "color":"#173177"
        },
        "keyword1":{
           "value":msg['code'],
           "color":"#173177"
        },
         "keyword2": {
             "value": msg['price'],
             "color": "#173177"
         },
        "keyword3": {
           "value":msg['total_deal_price'],
           "color":"#173177"
        },
        "keyword4": {
           "value":msg['quantity'],
           "color":"#173177"
        },
        "keyword5":{
           "value":msg['time'],
           "color":"#173177"
        },
        "remark":{
           "value":"请及时前往查看，谢谢！",
           "color":"#173177"
        }
    }
```
模板的数据格式可以在公众号自行定义，按照上面自己定义的模板消息。（提示：个人订阅号不能定制模板消息，需要服务号才可以。）
## 微信推送实例
> wx_push_test.py

运行这个程序，即可通过自己的公众号发送测试消息！
```
import requests
import json
import time
from Config import Config
from check_config import CheckConfig

RET_OK = 0
config = Config()
cc = CheckConfig()
ret, msg = cc.check_all()
if ret != RET_OK:
    print(ret, msg)
    exit()


class wechat_push(object):
    def __init__(self):
        self.appid = config.appid
        self.secrect = config.secrect
        self.access_generate_time = 0
        self.access_token = None

    # 获取accessToken
    def get_access_token_from_wechat(self):
        access_token_json = requests.get(
            url="https://api.weixin.qq.com/cgi-bin/token",
            params={
                "grant_type": "client_credential",
                "appid": self.appid,
                "secret": self.secrect
            }
        ).json()
        if(access_token_json):
            self.access_token = access_token_json['access_token']
        else:
            print(access_token_json)
            self.access_token = None
        return self.access_token

    def get_access_token(self):
        if(self.access_generate_time == 0 or time.time() - self.access_generate_time > 7200 - 5):
            self.access_token = self.get_access_token_from_wechat()
            self.access_generate_time = time.time()
            print("Generate access token.")
        return self.access_token

    def send_msg_to_users_customer_service_news(self, openid, msg):

        access_token = self.get_access_token()
        body = {
            "touser": openid,
            "msgtype": "text",
            "text": {
                "content": msg
            }
        }
        response = requests.post(
            url="https://api.weixin.qq.com/cgi-bin/message/custom/send",
            params={
                'access_token': access_token
            },
            data=bytes(json.dumps(body, ensure_ascii=False), encoding='utf-8')
        )

        result = response.json()
        print(result)

    def send_template_msg(self, openid, msg):

        access_token = self.get_access_token()
        body = {
            "touser": openid,
            "template_id":config.template_id,
             "data":{
                    "first": {
                       "value":(msg['echo_type']),
                       "color":"#173177"
                    },
                    "keyword1":{
                       "value":msg['code'],
                       "color":"#173177"
                    },
                     "keyword2": {
                         "value": msg['price'],
                         "color": "#173177"
                     },
                    "keyword3": {
                       "value":msg['total_deal_price'],
                       "color":"#173177"
                    },
                    "keyword4": {
                       "value":msg['quantity'],
                       "color":"#173177"
                    },
                    "keyword5":{
                       "value":msg['time'],
                       "color":"#173177"
                    },
                    "remark":{
                       "value":"请及时前往查看，谢谢！",
                       "color":"#173177"
                    }
               }
        }
        response = requests.post(
            url="https://api.weixin.qq.com/cgi-bin/message/template/send",
            params={
                'access_token': access_token
            },
            data=bytes(json.dumps(body, ensure_ascii=False), encoding='utf-8')
        )
        result = response.json()
        print(result)

    def get_user_openid_list(self):
        access_token = self.get_access_token()
        user_list_result = requests.get(
            url="https://api.weixin.qq.com/cgi-bin/user/get",
            params={
                'access_token': access_token
            }
        ).json()

        if user_list_result.get("total"):
            user_openid_list = user_list_result.get('data').get('openid')
        else:
            user_openid_list = None
        return user_openid_list

    def send_text_msg(self, msg):
        user_openid_list = self.get_user_openid_list()
        if user_openid_list:
            for openid in user_openid_list:
                self.send_msg_to_users_customer_service_news(openid, msg)

    def sent_template_msg_to_users(self, msg):
        user_openid_list = self.get_user_openid_list()
        if user_openid_list:
            for openid in user_openid_list:
                self.send_template_msg(openid, msg)


# ---- 单元测试代码
if __name__ == '__main__':
    msg = {
        'echo_type': 'Warning',
        'code':str(10000),
        'price': str(101.1),
        'total_deal_price':str(200),
        'quantity': str(20000),
        'time': str(123)
    }
    text_msg = 'Hello'

    wp = wechat_push()  # test号
    wp.send_text_msg(text_msg)
    wp.sent_template_msg_to_users(msg)

```


## 运行
> wx_push_test.py

成功运行时，终端显示：

![终端显示](https://upload-images.jianshu.io/upload_images/10171495-df73d096d78c5ecc.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)

![公众号回复](https://upload-images.jianshu.io/upload_images/10171495-3e20d8c209f5a46b.jpg?imageMogr2/auto-orient/strip%7CimageView2/2/w/500)

要注意的是，第一条消息的发送，调用的是客服消息接口，需要用户在近期内和公众号有交互，才可以成功发送。
